@@ -4,18 +4,19 @@
 import sys
 import trio
 import asyncio
-import warnings
 import threading
 from contextvars import ContextVar
 
-from async_generator import async_generator, yield_, asynccontextmanager
+try:
+    from contextlib import asynccontextmanager
+except ImportError:
+    from async_generator import asynccontextmanager
 
-from ._util import run_aio_future, run_aio_generator
 from ._async import TrioEventLoop
-from ._deprecate import deprecated, warn_deprecated
+from ._deprecate import warn_deprecated
 
 try:
-    from trio.hazmat import wait_for_child
+    from trio.lowlevel import wait_for_child
 except ImportError:
     from ._child import wait_for_child
 
@@ -100,7 +101,7 @@ _faked_policy = _FakedPolicy()
 
 def _in_trio_context():
     try:
-        trio.hazmat.current_task()
+        trio.lowlevel.current_task()
     except RuntimeError:
         return False
     else:
@@ -144,7 +145,7 @@ class _TrioPolicy(asyncio.events.BaseDefaultEventLoopPolicy):
         ``.current_event_loop`` property.
         """
         try:
-            task = trio.hazmat.current_task()
+            task = trio.lowlevel.current_task()
         except RuntimeError:
             pass
         else:
@@ -227,7 +228,7 @@ else:
 
     def _new_run_get():
         try:
-            task = trio.hazmat.current_task()
+            task = trio.lowlevel.current_task()
         except RuntimeError:
             pass
         else:
@@ -248,6 +249,8 @@ else:
 
     _aio_event._get_running_loop = _new_run_get
     _aio_event.get_running_loop = _new_run_get_or_throw
+    asyncio._get_running_loop = _new_run_get
+    asyncio.get_running_loop = _new_run_get_or_throw
 
 #####
 
@@ -341,7 +344,7 @@ class TrioChildWatcher(asyncio.AbstractChildWatcher if sys.platform != 'win32' e
 
     def add_child_handler(self, pid, callback, *args):
         """Add a callback to run when a child process terminates."""
-        h = self._loop.run_trio(self._waitpid, pid, callback, *args)
+        h = self._loop.trio_as_future(self._waitpid, pid, callback, *args)
         self._callbacks[pid] = h
 
     def remove_child_handler(self, pid):
@@ -368,7 +371,6 @@ class TrioChildWatcher(asyncio.AbstractChildWatcher if sys.platform != 'win32' e
 
 
 @asynccontextmanager
-@async_generator
 async def open_loop(queue_len=None):
     """Returns a Trio-flavored async context manager which provides
     an asyncio event loop running on top of Trio.
@@ -390,28 +392,43 @@ async def open_loop(queue_len=None):
 
     # TODO: make sure that there is no asyncio loop already running
 
-    def _main_loop_exit(self):
-        super()._main_loop_exit()
-        self._thread = None
-
-    async with trio.open_nursery() as nursery:
+    # The trio-asyncio loop can't shut down until all trio_as_aio tasks
+    # (or others using run_trio) have exited. This is because the
+    # termination of such a Trio task sets an asyncio future, which
+    # uses call_soon(), which won't work if the loop is closed.
+    # So, we use two nested nurseries.
+    async with trio.open_nursery() as loop_nursery:
         loop = TrioEventLoop(queue_len=queue_len)
         old_loop = current_loop.set(loop)
         try:
             loop._closed = False
-            await loop._main_loop_init(nursery)
-            await nursery.start(loop._main_loop)
-            await yield_(loop)
+            async with trio.open_nursery() as tasks_nursery:
+                await loop._main_loop_init(tasks_nursery)
+                await loop_nursery.start(loop._main_loop)
+                yield loop
+                tasks_nursery.cancel_scope.cancel()
+
+                # Allow all submitted run_trio() tasks calls a chance
+                # to start before the tasks_nursery closes, unless the
+                # loop stops (due to someone else calling stop())
+                # before that:
+                async with trio.open_nursery() as sync_nursery:
+                    sync_nursery.cancel_scope.shield = True
+
+                    @sync_nursery.start_soon
+                    async def wait_for_sync():
+                        if not loop.is_closed():
+                            await loop.synchronize()
+                        sync_nursery.cancel_scope.cancel()
+
+                    await loop.wait_stopped()
+                    sync_nursery.cancel_scope.cancel()
         finally:
             try:
-                await loop.stop().wait()
+                await loop._main_loop_exit()
             finally:
-                try:
-                    await loop._main_loop_exit()
-                finally:
-                    loop.close()
-                    nursery.cancel_scope.cancel()
-                    current_loop.reset(old_loop)
+                loop.close()
+                current_loop.reset(old_loop)
 
 
 def run(proc, *args, queue_len=None):
@@ -469,43 +486,3 @@ def run_trio_task(proc, *args):
     will propagate to, and terminate, the trio-asyncio loop.
     """
     _running_loop().run_trio_task(proc, *args)
-
-
-# These are aliases for methods in BaseTrioEventLoop which are
-# themselves deprecated. If we deprecate these and call the deprecated
-# loop method, the user gets two warnings. If we just call the
-# deprecated loop method, the warning points here instead of to user
-# code. Therefore, we "chase the pointer" and inline the body of the
-# deprecated loop method into each of these functions.
-
-
-@deprecated("0.10.0", issue=38, instead="aio_as_trio(proc(*args))")
-def wrap_generator(proc, *args):
-    return run_aio_generator(_running_loop(), proc(*args))
-
-
-@deprecated("0.10.0", issue=38, instead="aio_as_trio(aiter)")
-def run_iterator(aiter):
-    return run_aio_generator(_running_loop(), aiter)
-
-
-@deprecated("0.10.0", issue=38, instead="trio_as_aio(ctx)")
-def wrap_trio_context(ctx):
-    from ._adapter import Trio_Asyncio_Wrapper
-    return Trio_Asyncio_Wrapper(ctx, loop=_running_loop())
-
-
-@deprecated("0.10.0", issue=38, instead="aio_as_trio(proc)(*args)")
-def run_asyncio(proc, *args):
-    from ._adapter import Asyncio_Trio_Wrapper
-    return Asyncio_Trio_Wrapper(proc, args=args, loop=_running_loop())
-
-
-@deprecated("0.10.0", issue=38, instead="run_aio_coroutine")
-async def run_coroutine(fut):
-    return await _running_loop().run_aio_coroutine(fut)
-
-
-@deprecated("0.10.0", issue=38, instead="run_aio_future")
-async def run_future(fut):
-    return await run_aio_future(fut)
